@@ -46,7 +46,7 @@ An Ivy client is made of several threads:
 :group Objects and functions related to logging: ivylogger, debug, log, warn,
   error, ivy_loghdlr, ivy_logformatter
 
-Copyright (c) 2005-2008 Sebastien Bigaret <sbigaret@users.sourceforge.net>
+Copyright (c) 2005-2011 Sebastien Bigaret <sbigaret@users.sourceforge.net>
 """
 
 import logging, os
@@ -231,7 +231,7 @@ def decode_msg(msg):
     """
     
     :return: msg_id, numerical_id, parameters
-    :exception IvyMalformedMessage:
+    :except IvyMalformedMessage:
     """
     try:
         msg_id, _msg = msg.split(' ', 1)
@@ -345,7 +345,7 @@ class IvyClient:
         initial subscriptions for that client (or none); the initialization
         process completes after `end_init` is called.
         
-        :exception IvyIllegalStateError: if the method has already been called
+        :except IvyIllegalStateError: if the method has already been called
           once
         """
         if self.status != NOT_INITIALIZED:
@@ -358,7 +358,7 @@ class IvyClient:
         """
         Should be called when the initalization process ends.
         
-        :exception IvyIllegalStateError: if the method has already been called
+        :except IvyIllegalStateError: if the method has already been called
           (and ``self.status`` has already been set to ``INITIALIZED``)
         """
         if self.status is INITIALIZED:
@@ -368,7 +368,7 @@ class IvyClient:
         
     def add_regexp(self, regexp_id, regexp):
         """
-        :exception IvyIllegalStateError: if the client has not been fully
+        :except IvyIllegalStateError: if the client has not been fully
           initialized yet (see `start_init`)
         """
         if self.status not in ( INITIALIZATION_IN_PROGRESS, INITIALIZED ):
@@ -389,9 +389,9 @@ class IvyClient:
         Removes a regexp
         
         :return: the regexp that has been removed
-        :exception IvyIllegalStateError: if the client has not been fully
+        :except IvyIllegalStateError: if the client has not been fully
           initialized yet (see `start_init`)
-        :exception KeyError: if no such subscription exists
+        :except KeyError: if no such subscription exists
         """
         if self.status not in ( INITIALIZATION_IN_PROGRESS, INITIALIZED ):
             # initialization has not begun
@@ -565,7 +565,7 @@ class IvyClient:
         subscribers.
 
         .. note:: ``ivysocket.c:SocketSendRaw()`` also ignores error, simply
-          logging them.
+           logging them.
         
         """
         import socket
@@ -575,7 +575,7 @@ class IvyClient:
             log('[ignored] Error on socket with %r: %s', self, exc)
             
 import SocketServer, threading
-class IvyServer(SocketServer.ThreadingTCPServer, threading.Thread):
+class IvyServer(SocketServer.ThreadingTCPServer):
     """
     An Ivy server is responsible for receiving and handling the messages
     that other clients send on an Ivy bus to a given agent.
@@ -644,7 +644,7 @@ class IvyServer(SocketServer.ThreadingTCPServer, threading.Thread):
 
         :see: `bind_msg()`, `start()`
         """
-        threading.Thread.__init__(self, target=self.serve_forever)
+        self._thread = None
 
         # the empty string is equivalent to INADDR_ANY
         SocketServer.TCPServer.__init__(self, ('',0),IvyHandler)
@@ -673,7 +673,6 @@ class IvyServer(SocketServer.ThreadingTCPServer, threading.Thread):
         self._global_lock = threading.RLock()
 
         self.usesDaemons = usesDaemons
-        self.setDaemon(self.usesDaemons)
         self.server_termination = threading.Event()
 
         import time,random
@@ -705,19 +704,35 @@ class IvyServer(SocketServer.ThreadingTCPServer, threading.Thread):
     def start(self, ivybus=None):
         """
         Binds the server to the ivybus. The server remains connected until
-        `stop` is called, or 
+        `stop` is called, or until it receives and accepts a 'die' message.
+
+        :except IvyIllegalStateError: if the server has already been
+          started
         """
+        if self._thread is not None:
+          error('Cannot start: IvyServer already started')
+          raise IvyIllegalStateError('not running')
+
         self.ivybus = ivybus
         import socket
 
         log('Starting IvyServer on port %li', self.port)
-        threading.Thread.start(self)
+        self.server_termination.clear()
+        self._thread = threading.Thread(target=self.serve_forever)
+        self._thread.setDaemon(self.usesDaemons)
+        self._thread.start()
 
     def stop(self):
         """
         Disconnects the server from the ivybus. It also sets the
         `server_termination` event.
+
+        :except IvyIllegalStateError: if the server is not running started
         """
+        if not self.isAlive():
+          error('Cannot stop: not running')
+          raise IvyIllegalStateError('not running')
+
         self._global_lock.acquire()
         try:
             import socket
@@ -729,7 +744,14 @@ class IvyServer(SocketServer.ThreadingTCPServer, threading.Thread):
         finally:
             self._global_lock.release()
         self.server_termination.set()
+        self._thread.join()
+        self._thread = None
         
+    def isAlive(self):
+      if self._thread is None:
+        return False
+      return self._thread.isAlive()
+
     def get_clients(self):
         """
         Returns the list of the agent names of all connected clients
@@ -908,7 +930,7 @@ class IvyServer(SocketServer.ThreadingTCPServer, threading.Thread):
         Unregisters the corresponding regexp
 
         .. warning:: this method is not MT-safe, callers must acquire the
-          global lock
+           global lock
 
         :return: the regexp that has been removed
         :except KeyError: if no such subscription can be found
@@ -919,7 +941,7 @@ class IvyServer(SocketServer.ThreadingTCPServer, threading.Thread):
         """
 
         .. warning:: this method is not MT-safe, callers must acquire the
-          global lock
+           global lock
         """
         return self._subscriptions[int(idx)][1]
 
